"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.printOutboxListenerEnvVariables = exports.printInboxListenerEnvVariables = exports.getOutboxListenerSettings = exports.getInboxListenerSettings = exports.outboxEnvPrefix = exports.inboxEnvPrefix = exports.fallbackEnvPrefix = void 0;
const env_settings_1 = require("./env-settings");
const basicSettingsMap = [
    {
        constantName: 'DB_SCHEMA',
        default: 'public',
        func: env_settings_1.getEnvVariableString,
    },
    {
        constantName: 'MESSAGE_PROCESSING_TIMEOUT_IN_MS',
        default: 15000,
        func: env_settings_1.getEnvVariableNumber,
    },
    { constantName: 'MAX_ATTEMPTS', func: env_settings_1.getEnvVariableNumber, default: 5 },
    {
        constantName: 'MAX_POISONOUS_ATTEMPTS',
        default: 3,
        func: env_settings_1.getEnvVariableNumber,
    },
    {
        constantName: 'MESSAGE_CLEANUP_INTERVAL_IN_MS',
        default: 5 * 60 * 1000,
        func: env_settings_1.getEnvVariableNumber,
    },
    {
        constantName: 'MESSAGE_CLEANUP_PROCESSED_IN_SEC',
        default: 7 * 24 * 60 * 60,
        func: env_settings_1.getEnvVariableNumber,
    },
    {
        constantName: 'MESSAGE_CLEANUP_ABANDONED_IN_SEC',
        default: 14 * 24 * 60 * 60,
        func: env_settings_1.getEnvVariableNumber,
    },
    {
        constantName: 'MESSAGE_CLEANUP_ALL_IN_SEC',
        default: 60 * 24 * 60 * 60,
        func: env_settings_1.getEnvVariableNumber,
    },
];
const inboxSettingsMap = [
    {
        constantName: 'DB_TABLE',
        default: 'inbox',
        func: env_settings_1.getEnvVariableString,
    },
    {
        constantName: 'ENABLE_MAX_ATTEMPTS_PROTECTION',
        default: true,
        func: env_settings_1.getEnvVariableBoolean,
    },
    {
        constantName: 'ENABLE_POISONOUS_MESSAGE_PROTECTION',
        default: true,
        func: env_settings_1.getEnvVariableBoolean,
    },
];
const outboxSettingsMap = [
    {
        constantName: 'DB_TABLE',
        default: 'outbox',
        func: env_settings_1.getEnvVariableString,
    },
    {
        constantName: 'ENABLE_MAX_ATTEMPTS_PROTECTION',
        default: false,
        func: env_settings_1.getEnvVariableBoolean,
    },
    {
        constantName: 'ENABLE_POISONOUS_MESSAGE_PROTECTION',
        default: false,
        func: env_settings_1.getEnvVariableBoolean,
    },
];
exports.fallbackEnvPrefix = 'TRX_';
exports.inboxEnvPrefix = 'TRX_INBOX_';
exports.outboxEnvPrefix = 'TRX_OUTBOX_';
/**
 * Loads the environment variables into the listener settings object. It
 * supports reading an inbox specific setting or a general one.
 * Please use the `printXxxxListenerEnvVariables` functions to get a list of all
 * the outbox and inbox relevant settings for the replication or polling
 * listener.
 * @example
 * TRX_DB_SCHEMA=trx_schema
 * TRX_INBOX_DB_TABLE=inbox_table
 * TRX_MESSAGE_PROCESSING_TIMEOUT_IN_MS=30000
 * @param env The process.env variable or a custom object.
 * @returns The listener settings object with filled with the ENV variables
 */
const getInboxListenerSettings = (env = process.env) => (0, env_settings_1.getConfigSettings)([...basicSettingsMap, ...inboxSettingsMap], exports.inboxEnvPrefix, exports.fallbackEnvPrefix, env);
exports.getInboxListenerSettings = getInboxListenerSettings;
/**
 * Loads the environment variables into the listener settings object. It
 * supports reading an outbox specific setting or a general one.
 * Please use the `printXxxxListenerEnvVariables` functions to get a list of all
 * the outbox and outbox relevant settings for the replication or polling
 * listener.
 * @example
 * TRX_DB_SCHEMA=trx_schema
 * TRX_OUTBOX_DB_TABLE=outbox_table
 * TRX_MESSAGE_PROCESSING_TIMEOUT_IN_MS=30000
 * @param env The process.env variable or a custom object.
 * @returns The listener settings object with filled with the ENV variables
 */
const getOutboxListenerSettings = (env = process.env) => (0, env_settings_1.getConfigSettings)([...basicSettingsMap, ...outboxSettingsMap], exports.outboxEnvPrefix, exports.fallbackEnvPrefix, env);
exports.getOutboxListenerSettings = getOutboxListenerSettings;
/**
 * Prints the available env variables and their default values for the basic
 * inbox listener env variables.
 */
const printInboxListenerEnvVariables = () => (0, env_settings_1.printConfigSettings)([...basicSettingsMap, ...inboxSettingsMap], exports.inboxEnvPrefix, exports.fallbackEnvPrefix);
exports.printInboxListenerEnvVariables = printInboxListenerEnvVariables;
/**
 * Prints the available env variables and their default values for the basic
 * outbox listener env variables.
 */
const printOutboxListenerEnvVariables = () => (0, env_settings_1.printConfigSettings)([...basicSettingsMap, ...outboxSettingsMap], exports.outboxEnvPrefix, exports.fallbackEnvPrefix);
exports.printOutboxListenerEnvVariables = printOutboxListenerEnvVariables;
//# sourceMappingURL=listener-config.js.map